/*
 * This file will handle all ui specific code for the image manipulation2.
 * 
 */
require(["aloha/jquery","aloha/pluginmanager", "aloha/floatingmenu"],
	function(aQuery, PluginManager, FloatingMenu) {
	
	var	jQuery = aQuery;
	var $ = aQuery;
	
	var ImageManipulation = {
		
			urlParams: null,
			
			init: function() {

				var that = this;

				//Workaround since event handling of sidebar is not working properly at the moment
				this.populateSidebar();
				
				// Handle save action for the current image	
				$('#save-action').click(jQuery.proxy(that.handleImageSaveAction, that));
	
				$('body').bind('aloha-image-cropped', function(e, image, props) {
					$("#imageSize").text($(image).height() + "x" + $(image).width() );
					if(!$('#image-actions').is(":visible")) {
						$('#image-actions').slideDown('slow');
					}					
				});
				
				$('body').bind('aloha-image-reset', function(e, image) {
					
					// Hide the actions menu
					if( $('#image-actions').is(":visible") ) {
						$('#image-actions').slideUp('slow');
					}
				});
				
				// Handle the resize event and update some information about the image
				$('body').bind('aloha-image-resize', function(e, image) {
					// Show the actions menu
					$("#imageSize").text($(image).height() + "x" + $(image).width() );
					if(!$('#image-actions').is(":visible")) {
						$('#image-actions').slideDown('slow');
					}					
				});
				
				// Handle the image resized event and update the coordinates
				$('body').bind('aloha-image-resized', function(e, image) {
					$("#imageSize").text($(image).height() + "x" + $(image).width() );
				});
				
				$('body').bind('aloha-gcn-image-copied', function(e, imageProperties) {
					that.showMessageBox('Copy of image was successfully created.', 4000);
					that.populateImagePropertyFields(imageProperties);
				});
				
				// Handle the image updated event and populate all field with the data
				$('body').bind('aloha-gcn-image-updated', function(e, imageProperties) {
					that.populateImagePropertyFields(imageProperties);
					var image = PluginManager.plugins['image'].imageObj;
					$("#imageSize").text($(image).height() + "x" + $(image).width() );
				});

			},
			
			
			/**
			 * This method will handle the image save action
			 */
			handleImageSaveAction: function() {
				var that = this;
				var imageProperties = PluginManager.plugins['gcn-image'].imageProperties;
				imageProperties.name = $('#filename').val();
				imageProperties.description =$('#description').val();  
							
				/**
				 * Handles errors that might happen during saving of the image
				 */
				function onError(jqXHR, textStatus, errorThrown) {
					//TODO i18n this
					that.showMessageBox('Could not save image properties', -1, 'warning');
				};
				
				/**
				 * Shows a status message and invokes the next rest call
				 */
				function onSuccess(data) {
					that.showMessageBox('Saved image properties', 3000);
					
					var imageProperties = PluginManager.plugins['gcn-image'].imageProperties;
					
					// Saving was successful now update the image properties
					//imageProperties = jQuery.extend(true, imageProperties, imageProperties.image);
					//PluginManager.plugins['gcn-image'].imageProperties = imageProperties;
					
					// We need to invoke this method since the filename might has been changed due to filename conflict checks.
					// Furthermore the rest response currently does not contain a file/image object.
					PluginManager.plugins['gcn-image'].updateImageProperties(imageProperties.id, function(loadedImageProperties) {

						that.populateImagePropertyFields(loadedImageProperties);
						
						// Update the properties within the tagfill frame
						if (window.opener && typeof window.opener.updateFileField  === "function") {
							window.opener.updateFileField(loadedImageProperties.name, loadedImageProperties.id);
						}
						
					});
					
					
					function onError(jqXHR, textStatus, errorThrown) {
						//TODO i18n this
						that.showMessageBox('Error while resizing image', -1, 'warning');
					};
					
					function onSuccess(data) {
						that.showMessageBox('Resized image saved.', 400);
						
						// Finally we can close this window since all rest calls were successful 
						// and this window was opened from a tagfill window
						if (window.opener && typeof window.opener.updateFileField  === "function") {
							window.close();
							return;
						}
						
						//Restore the assistant frame before closing the window
						that.restoreFrameUI();
						window.history.back();
						window.history.back();

					};
					
					//TODO Check if resize is needed or if the user has just changed the filename.
					PluginManager.plugins['gcn-image'].invokeImageResizeRestCall(imageProperties, onSuccess, onError);

				};
				
				PluginManager.plugins['gcn-image'].invokeImageSavePropertiesRestCall(imageProperties, onSuccess, onError);
			},
			
			/**
			 * Populates the fields with image properties
			 */
			populateImagePropertyFields: function(imageProperties) {
				$('#imageTitle').text(imageProperties.name);
				$('#description').val(imageProperties.text);
				$('#filename').val(imageProperties.name);	
			},
			
			/**
			 * Adds panels to the sidebar
			 */
			populateSidebar: function() {
				
				var that = this;
				
				var left = Aloha.Sidebars.left;

				left.addPanel({
					title	 : 'Properties',
					content  : $('#panelLeftPropertiesContainer').remove().html(),
					expanded : true,
					activeOn : true,
					onInit	 : that.loadImageProperties
				});
				
				// Disabling upload panel since it will be added in phase 2
				/*
				left.addPanel({
					title    : 'Upload',
					content  : $('#panelLeftUploadContainer').remove().html(),
					expanded : false,
					activeOn :  true,
					onInit   : function () {
						var that = this;
					}
				});
				*/				
				left.show();
		
			},

			/**
			 * Shows a small message box at the top of the window. The box will stay for the given amount of time. 
			 * A given duration of -1 will ommit the slideup action. 
			 */
			showMessageBox: function(text, duration, type) {
				
				// If no duration is specified we'll use a default amount	
				if (typeof duration === "undefined") {
					duration = 3000;
				}
				
				function hideMessageBox() {
					$('#message-container').slideUp('slow');
				}
				
				$('#message-container').removeClass();
				$('#message-image').removeClass();
				
				if (typeof type !== "undefined") {
					$('#message-container').addClass(type);
					$('#message-image').addClass(type);
				}
				$('#message-container span').text(text);
				$('#message-container').slideDown('slow', function() {
					if (duration!=-1) {
						setTimeout(hideMessageBox, duration);
					}
				});
			},

			/**
			 * Get the assistant frame as jQuery object (if editor is opened in the GCN Frameset). Otherwise returns false.
			 * @return assistant frame as jQuery object or false
			 */
			getAssistantFrame: function() {
				// only get the assistant frame once
				if (this.assistantFrame !== undefined) {
					return this.assistantFrame;
				}
				if (!window.parent) {
					return false;
				}
				
				// check for the vertical assistant
				var assistant = jQuery('frame', window.parent.document).eq(0);
				if (assistant.attr('name') === 'ass') {
					this.assistantFrame = assistant;
					return assistant;
				}
				
				// check for the horizontal assistant
				assistant = jQuery('frame', window.parent.document).eq(2);
				if (assistant.attr('name') === 'ass') {
					this.assistantFrame = assistant;
					return assistant;
				}
			
				// we could also be in an iframe, so check further
				if (!window.parent.parent) {
					return false;
				}
				
				// check for the vertical assistant
				var assistant = jQuery('frame', window.parent.parent.document).eq(0);
				if (assistant.attr('name') === 'ass') {
					this.assistantFrame = assistant;
					return assistant;
				}
				
				// check for the horizontal assistant
				assistant = jQuery('frame', window.parent.parent.document).eq(2);
				if (assistant.attr('name') === 'ass') {
					this.assistantFrame = assistant;
					return assistant;
				}
			
				return false; 
			},
			
			/**
			 * Removes the GCN assistant frame and removes the content from the main menu frame.
			 * This function also stops the GCN loading bar.
			 * 
			 * It's save to call this function even when not running inside the GCN frame.
			 * In this case the function will do nothing.
			 * 
			 * @return void
			 */
			updateFrameUI: function () {
				var assistantFrame = this.getAssistantFrame();
				if (assistantFrame) {
					var assistantFrameset = assistantFrame.parent();
					this.originalColumns = assistantFrameset.attr('cols');
					this.originalRows = assistantFrameset.attr('rows');
					if (this.originalColumns) {
						assistantFrameset.attr('cols', '0,*,0');
					}
					if (this.originalRows) {
						assistantFrameset.attr('rows', '*,0,0');
					}
			
					var menuFrame = jQuery(window.parent.parent.frames[2].document);
					// hide the progress bar
					menuFrame.find('#ProgressStatusInfo_0').hide();
				}
			},
			
			/**
			 * Restores the GCN UI so that it can be used like before.
			 * 
			 * It's save to call this function even when not running inside the GCN frame.
			 * In this case the function will do nothing.
			 * 
			 * @return void
			 */
			restoreFrameUI: function () {
				var assistant = this.getAssistantFrame();
				if (assistant) {
					var assistantFrameset = assistant.parent();
					if (this.originalColumns) {
						assistantFrameset.attr('cols', this.originalColumns);
					}
					if (this.originalRows) {
						assistantFrameset.attr('rows', this.originalRows);
					}
				}
			}
			
	};
	
	
	require.ready(function() {
	
			var	$ = aQuery;
			var $body = $('body');
			
			
			$('#message-container').click( function() {
				$(this).slideUp();
			});

			
			// Hide the assistant
			ImageManipulation.updateFrameUI();

			// Move the floatingmenu to the left when the sidebar might cover it
			$body.bind('aloha-sidebar-opened', function (ev, sidebar) {
									
				if(FloatingMenu.left < sidebar.width) {
					FloatingMenu.floatTo({
						top : FloatingMenu.top,
						left : FloatingMenu.left+250
					});
				}
			});
				
			// Update the container image to display the initial image
				
			$body.bind('aloha',function () {

				ImageManipulation.init();

				// Trigger click on the image to enable resize
				$('img').filter('.global').trigger('click');
					$('#message-container').slideUp('slow');
				});
				
				// Change text after core is loaded 
//				$body.bind('alohacoreloaded',function(){
//                 $body.trigger('aloha-sidebar-ready');
//				});
					
	});
	
});

